from charmhelpers.core.hookenv import (
    action_get,
    action_fail,
    action_set,
    status_set,
)

from charms.reactive import (
    clear_flag,
    set_flag,
    when,
    when_not,
)
import charms.sshproxy
import json
import random
import ipaddress

SUPPORTED_PROTO = ['tcp', 'udp', 'icmp', 'icmpv6']
@when('sshproxy.configured')
@when_not('srv6.installed')
def install_srv6():
    # Do your setup here.
    #
    # If your charm has other dependencies before it can install,
    # add those as @when() clauses above., or as additional @when()
    # decorated handlers below
    #
    # See the following for information about reactive charms:
    #
    #  * https://jujucharms.com/docs/devel/developer-getting-started
    #  * https://github.com/juju-solutions/layer-basic#overview
    #
    set_flag('srv6.installed')

    # Tell's the VCA that the charm is ready to be used.
    status_set('active', 'Ready!')

@when('actions.create-chain')
def chain():
    """Create a new SRv6 chain in inline mode."""
    err = ''
    try:
        newChain={}
        newChain['segList'] = action_get('list')
        newChain['destination'] = destination=str(ipaddress.IPv6Address( action_get('dest') ))
        newChain['protocol']= action_get('proto')
        newChain['source'] = action_get('source')
        newChain['sport']= action_get('port-source')
        newChain['dport'] = action_get('port-dest')
        newChain['tablen']=str(random.randint(100,250))
        newChain['tableid']='localsid'+newChain['tablen']
        newChain['markn']=str(random.randint(1,100))
        with open('chainCreated.json') as f:
            chainCreated = json.load(f)
        cmd=''

        if newChain['destination'] and newChain['segList']:
            #TODO Check for existing chain with same param
            cmdTable = ['sudo echo {} {} | sudo tee -a /etc/iproute2/rt_tables'.format(newChain['tablen'], newChain['tableid'])]
            result, err = charms.sshproxy._run(cmdTable)


            cmdRule = ['sudo ip -6 rule add fwmark {} table {}'.format(newChain['markn'], newChain['tableid'])]
            result, err = charms.sshproxy._run(cmdRule)

            cmdSrv6 = ['sudo ip -6 route add {} encap seg6 mode inline segs {} dev ens3 table {}'.format(newChain['destination'], newChain['segList'], newChain['tableid'])]
            result, err = charms.sshproxy._run(cmdSrv6)

            cmdIptables = ['sudo ip6tables -6 -A OUTPUT  -t mangle -d {}'.format(newChain['destination'])]

            if newChain['source']:
                cmdIptables[0]= cmdIptables[0]+ ' -s '+ newChain['source']
            if newChain['protocol'] in SUPPORTED_PROTO:
                cmdIptables[0]= cmdIptables[0]+ ' -p '+ newChain['protocol']
            if newChain['protocol'] == 'tcp' or newChain['protocol'] == 'udp':
                if newChain['sport']:
                    cmdIptables[0]= cmdIptables[0]+ ' --sport '+ newChain['sport']
                if newChain['dport']:
                    cmdIptables[0]= cmdIptables[0]+ ' --dport '+ newChain['dport']
            cmdIptables[0]= cmdIptables[0]+' -j MARK --set-mark '+newChain['markn']
            result, err = charms.sshproxy._run(cmdIptables)
            cmdIptables[0]= cmdIptables[0].replace('OUTPUT','POSTROUTING')
            result, err = charms.sshproxy._run(cmdIptables)
            chainCreated['chain_list'].append(newChain)
            with open('chainCreated.json', 'w') as f:
                f.truncate(0)
                json.dump(chainCreated,f)

    except:
        action_fail('command failed:' + err)
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.create-chain')

@when('actions.update-chain')
def chain():
    """Update the segment list of an existing chain."""
    err = ''
    try:
        segList = action_get('list')
        destination=str(ipaddress.IPv6Address( action_get('dest') ))
        with open('chainCreated.json') as f:
            chainCreated = json.load(f)
        cmd=''
        for chain in chainCreated['chain_list']:
            if chain['destination'] == destination:
                tableid=chain['tableid']
                cmdSrv6Del = ['sudo ip -6 route del {} table {}'.format(destination,tableid)]
                result, err = charms.sshproxy._run(cmdSrv6Del)
                cmdSrv6 = ['sudo ip -6 route add {} encap seg6 mode inline segs {} dev ens3 table {}'.format(destination, segList,tableid)]
                result, err = charms.sshproxy._run(cmdSrv6)
                chainCreated['chain_list'].remove(chain)
                chain['segList']=segList
                chainCreated['chain_list'].append(chain)
                with open('chainCreated.json', 'w') as f:
                    f.truncate(0)
                    json.dump(chainCreated,f)
                break
    except:
        action_fail('command failed:' + err)
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.update-chain')

@when('actions.del-chain')
def delChain():
    """Delete an existing Srv6 chain."""
    err = ''
    try:
        destination=str(ipaddress.IPv6Address( action_get('dest') ))
        with open('chainCreated.json') as f:
            chainCreated = json.load(f)
        for i in chainCreated['chain_list']:
            if i['destination'] == destination:
                markn = i['markn']
                tableid=i['tableid']
                cmdRuleDel = ['sudo ip -6 rule del fwmark {} table {}'.format(markn,tableid)]
                result, err = charms.sshproxy._run(cmdRuleDel)
                cmdSrv6Del = ['sudo ip -6 route del {}  table {}'.format(destination,tableid)]
                result, err = charms.sshproxy._run(cmdSrv6Del)
                cmdIptables = ['sudo ip6tables -6 -t mangle -D OUTPUT -d {}'.format(destination)]

                if i['source']:
                    cmdIptables[0]= cmdIptables[0]+ ' -s '+ i['source']
                if i['protocol'] in SUPPORTED_PROTO:
                    cmdIptables[0]= cmdIptables[0]+ ' -p '+ i['protocol']
                if i['protocol'] == 'tcp' or i['protocol'] == 'udp':
                    if i['sport']:
                        cmdIptables[0]= cmdIptables[0]+ ' --sport '+ i['sport']
                    if i['dport']:
                        cmdIptables[0]= cmdIptables[0]+ ' --dport '+ i['dport']
                cmdIptables[0]= cmdIptables[0]+' -j MARK --set-mark '+i['markn']
                cmdIptables[0]= cmdIptables[0]+';'+cmdIptables[0].replace('OUTPUT','POSTROUTING')
                result, err = charms.sshproxy._run(cmdIptables)

                cmdTable=["sudo sed -i -e '/{}/d'  /etc/iproute2/rt_tables".format(i['tablen'])]
                result, err = charms.sshproxy._run(cmdTable)

                chainCreated['chain_list'].remove(i)
                with open('chainCreated.json', 'w') as f:
                    f.truncate(0)
                    json.dump(chainCreated,f)
    except:
        action_fail('command failed:' + err)
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.del-chain')

#Old function to be improved, not realiable yet...

@when('actions.encap-chain')
def chain():
    """chain a file."""
    err = ''
    try:
        segList = action_get('list')
        destination = action_get('dest')
        cmd = ['sudo ip -6 route add {} encap seg6 mode encap segs {} dev ens3'.format(destination, segList)]
        result, err = charms.sshproxy._run(cmd)
    except:
        action_fail('command failed:' + err)
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.encap-chain')


@when('actions.end')
def endC():
    """chain a file."""
    err = ''
    try:
        via = action_get('via')
        destination = action_get('dest')
        if via:
            cmd = ['sudo ip -6 route add {} encap seg6local action End via {}'.format(destination, via)]
        else:
            cmd = ['sudo ip -6 route add {} encap seg6local action End dev ens3'.format(destination)]
        result, err = charms.sshproxy._run(cmd)
    except:
        action_fail('command failed:' + err)
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.end')

@when('actions.end-x')
def endX():
    """chain a file."""
    err = ''
    try:
        via = action_get('via')
        next = action_get('next')
        destination = action_get('dest')
        cmd = ['sudo ip -6 route add {} encap seg6local action End.X nh6 {} via {}'.format(destination, next, via)]
#        cmd = ['sudo ip -6 route add {} encap seg6local action End.X nh6 {} dev ens3'.format(destination, next)]
        result, err = charms.sshproxy._run(cmd)
    except:
        action_fail('command failed:' + err)
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.end-x')

@when('actions.end-t')
def endT():
    """chain a file."""
    err = ''
    try:
        via = action_get('via')
        table = action_get('table')
        destination = action_get('dest')
        cmd = ['sudo ip -6 route add {} encap seg6local action End.T table {} via {}'.format(destination, table, via)]
#        cmd = ['sudo ip -6 route add {} encap seg6local action End.T table {} dev ens3'.format(destination, next)]
        result, err = charms.sshproxy._run(cmd)
    except:
        action_fail('command failed:' + err)
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.end-t')

@when('actions.end-dx2')
def endDX2():
    """chain a file."""
    err = ''
    try:
        via = action_get('via')
        oif = action_get('oif')
        destination = action_get('dest')
        cmd = ['sudo ip -6 route add {} encap seg6local action End.DX2 oif {} via {}'.format(destination, oif, via)]
#        cmd = ['sudo ip -6 route add {} encap seg6local action End.DX2 oif {} dev ens3'.format(destination, next)]
        result, err = charms.sshproxy._run(cmd)
    except:
        action_fail('command failed:' + err)
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.end-dx2')

@when('actions.end-dx6')
def endDX6():
    """chain a file."""
    err = ''
    try:
        via = action_get('via')
        next = action_get('next')
        destination = action_get('dest')
        cmd = ['sudo ip -6 route add {} encap seg6local action End.DX6 nh6 {} via {}'.format(destination, next, via)]
#        cmd = ['sudo ip -6 route add {} encap seg6local action End.DX6 nh6 {} dev ens3'.format(destination, next)]
        result, err = charms.sshproxy._run(cmd)
    except:
        action_fail('command failed:' + err)
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.end-dx6')

@when('actions.end-dx4')
def endDX4():
    """chain a file."""
    err = ''
    try:
        via = action_get('via')
        next = action_get('next')
        destination = action_get('dest')
        cmd = ['sudo ip -6 route add {} encap seg6local action End.DX4 nh4 {} via {}'.format(destination, next, via)]
#        cmd = ['sudo ip -6 route add {} encap seg6local action End.DX4 nh4 {} dev ens3'.format(destination, next)]
        result, err = charms.sshproxy._run(cmd)
    except:
        action_fail('command failed:' + err)
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.end-dx4')


@when('actions.end-dt6')
def endDT6():
    """chain a file."""
    err = ''
    try:
        via = action_get('via')
        table = action_get('table')
        destination = action_get('dest')
        cmd = ['sudo ip -6 route add {} encap seg6local action End.DT6 table {} via {}'.format(destination, table, via)]
#        cmd = ['sudo ip -6 route add {} encap seg6local action End.DT6 table {} dev ens3'.format(destination, table)]
        result, err = charms.sshproxy._run(cmd)
    except:
        action_fail('command failed:' + err)
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.end-dt6')


@when('actions.end-b6')
def endB6():
    """chain a file."""
    err = ''
    try:
        via = action_get('via')
        segList = action_get('list')
        destination = action_get('dest')
        cmd = ['sudo ip -6 route add {} encap seg6local action End.B6 srh segs {} via {}'.format(destination, segList, via)]
#        cmd = ['sudo ip -6 route add {} encap seg6local action End.B6 srh segs {} dev ens3'.format(destination, segList)]
        result, err = charms.sshproxy._run(cmd)
    except:
        action_fail('command failed:' + err)
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.end-b6')

@when('actions.end-b6-en')
def endB6En():
    """chain a file."""
    err = ''
    try:
        via = action_get('via')
        segList = action_get('list')
        destination = action_get('dest')
        cmd = ['sudo ip -6 route add {} encap seg6local action End.B6.Encaps srh segs {} via {}'.format(destination, segList, via)]
#        cmd = ['sudo ip -6 route add {} encap seg6local action End.B6.Encaps srh segs {} dev ens3'.format(destination, segList)]
        result, err = charms.sshproxy._run(cmd)
    except:
        action_fail('command failed:' + err)
    else:
        action_set({'output': result})
    finally:
        clear_flag('actions.end-b6-en')
